/* nimrodg-schedule-organise.c: It sorts Nimrod-G schedule logs and organise them. 

-----
The Nimrod scheduler stores schedule logs in file "schedule.sched".
Format: UnixTimestamp  server-name No-of-Jobs-In-Exe Jobs_finished_so_far Budget_spent ..so on for each server/resource.

To get the value of seconds since starting of experiement:  (CurrentTimeStamp-FirstTimeStamp)

This program sorts values as per server name and then prints them.
It also creates three different files and stores scheduling data related
to each category of data (schedule.Fin for Jobs_finished_so_far for all servers in order.
-----

   Programmed by R. Buyya on Dec. 16, 2001. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details @ www.gnu.org
*/

#include <stdio.h>

/* Job Scheduling Data on Resources */
typedef struct ResSchedData
{
	int jobs_in_exec;	/* no of jobs in execution */
   int jobs_finished; /* no of jobs finished */
   float budget_spent;  /* budget spend for processing jobs finished so far */
} ResSchedData;

void ExtraFileNameWithoutExt( char *FileFullName, char *FileName );

int main( int argc, char *argv[] )
{
	int i;
	char sched_filename[100];
	char jobInExecFilename[100];
	char jobFinishedFilename[100];
	char budgetSpentFilename[100];
	FILE *fp; /* Pointer to Nimrod-G schedule data */
	FILE *job_in_exec; /* Nimdod-G Job_in_exec */
	FILE *job_finished; /* Jobs finished so far */
	FILE *budget_spent; /* Budget Spent to finish jobs so far */

	char timestamp[100]; 
	char buff[100];
	
	int resource_count;
	int no_more_records = 0;
	int res_id;  /* if server/resource name is "s1", then id = 1 */
	char res_prefix; /* if server name is "s1", then prefix is "s" */
	ResSchedData *sched;
	ResSchedData sd;

	if( argc == 1 )
	{
		printf("Usage: %s <Nimrod-G-ScheduleFile>\n", argv[0]);
		printf("The first line of  schedule data file should contain No-Of-Resources (integer) Used.\n");
		return 0;
	}

	if( (fp= fopen( argv[1], "r")) == NULL )
	{
		printf("Unable to open Nimrod-G schedule File: %s: %s\n", argv[1]);;
		exit(1);
	}

	ExtraFileNameWithoutExt(argv[1], sched_filename );
	sprintf( jobInExecFilename, "%s.InExec", sched_filename );
	sprintf( jobFinishedFilename, "%s.Fin", sched_filename );
	sprintf( budgetSpentFilename, "%s.Spent", sched_filename );

	if( (job_in_exec = fopen(jobInExecFilename, "w")) == NULL )
	{
		printf("Unable to open Nimrod-G schedule Output File: %s: %s\n", jobInExecFilename);
		exit(1);
	}

	if( (job_finished = fopen(jobFinishedFilename, "w")) == NULL )
	{
		printf("Unable to open Nimrod-G schedule Output File: %s: %s\n", jobFinishedFilename);
		exit(1);
	}

	if( (budget_spent = fopen(budgetSpentFilename, "w")) == NULL )
	{
		printf("Unable to open Nimrod-G schedule Output File: %s: %s\n", budgetSpentFilename);
		exit(1);
	}

	fscanf(fp, "%d", &resource_count);
	sched = (ResSchedData *) malloc(sizeof(ResSchedData)*resource_count);

	while(!feof(fp) )
	{
		/* First read timestamp */
		fscanf(fp, "%s", timestamp);
		if( feof(fp) )
			break;

		for(i = 0; i < resource_count; i++)
		{
			fscanf(fp, "%s", buff );
			sscanf(buff, "%c%d", &res_prefix, &res_id);
			fscanf(fp, "%d", &sd.jobs_in_exec );
			fscanf(fp, "%d", &sd.jobs_finished );
			fscanf(fp, "%f", &sd.budget_spent );

			sched[res_id-1].jobs_in_exec = sd.jobs_in_exec;
			sched[res_id-1].jobs_finished = sd.jobs_finished;
			sched[res_id-1].budget_spent = sd.budget_spent;
		}

		printf("%s", timestamp);
		fprintf(job_in_exec, "%s", timestamp );
		fprintf(job_finished, "%s", timestamp );
		fprintf(budget_spent, "%s", timestamp );

		for(i = 0; i < resource_count; i++)
		{
			printf("  %d %d %f", sched[i].jobs_in_exec, sched[i].jobs_finished, sched[i].budget_spent );
			fprintf(job_in_exec, "  %d", sched[i].jobs_in_exec );
			fprintf(job_finished,"  %d", sched[i].jobs_finished );
			fprintf(budget_spent,"  %f", sched[i].budget_spent );
		}
		printf("\n");
		fprintf(job_in_exec, "\n");
		fprintf(job_finished, "\n");
		fprintf(budget_spent, "\n");
	}
}

/* Extract FileName execluding extension from the FileFullName ) */
void ExtraFileNameWithoutExt( char *FileFullName, char *FileName )
{
	int i;
	for(i = 0; FileFullName[i]; i++)
	{
		if(FileFullName[i] == '.')
			break;
		FileName[i] = FileFullName[i];
	}
	FileName[i]='\0';
}
