/* request-dispatch.c: server request dispatcher. It accepts clients commands and
   dispatches requests to appropriate modules to process and respond to clients.

    Copyright (C) 2001-2099, Rajkumar Buyya (rajkumar@buyya.com)!

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details @ www.gnu.org
*/

#include "header.h"
extern int cdb_server_verbose;

char *services[] = { "Echo", "close", "GetFile", "PutFile", "GetMoleculeRec", "shutdown", NULL };

void echo_message( int handle );

enum COMMANDS { ECHO, CONNECTION_CLOSE, GETFILE, PUTFILE, GET_MOLECULE_REC, SHUTDOWN };
int MsgToCommand( char *msg )
{
   int i;
   for( i = 0; services[i]; i++ )
      if( !strcmp(services[i], msg) )
         return i; /* service supported */
   return -1;
}

void *service_dispatch(int *client_socket_addr)
{
   char command[100];
   int service_type, status;
	int client_socket;

	AllocateWorker();

	client_socket = *client_socket_addr;

   if( readline( client_socket, command, 100 ) != -1 )
   {

		/* some processing here. servicing to a client request based on input data */
		service_type = MsgToCommand( command );

		/* 
  		status = globus_module_activate(GLOBUS_GASS_FILE_MODULE); 
		if( status == GLOBUS_SUCCESS )
			printf("active successful"\n");
   	else
			printf("active Unsuccessful"\n");
		*/
		switch( service_type )
   	{
			case CONNECTION_CLOSE:
				break;
			case ECHO:
				echo_message( client_socket );
				break;
			case GETFILE:
				status = ProxyGetFile( client_socket );
				break;
			case PUTFILE:
				status = ProxyPutFile( client_socket );
				break;
			case GET_MOLECULE_REC:
				status = GetMoleculeRequest( client_socket );
				break;
			case SHUTDOWN:
				pserver_down( client_socket );
				break;
		} 
 /*  globus_module_deactivate(GLOBUS_GASS_FILE_MODULE); */
   }
   else
		printf("Client is not responding OR has not sent any request.\n");

	FreeWorker();

   /* close connection */
   close( client_socket ); 
   pthread_exit( (void *)0); 
}

void echo_message( int handle )
{
	int i;
	char buff[100];
	while(1)
	{
		readline( handle, buff, 100 );
		printf("%s\n", buff);
		for( i = 0; i < strlen(buff); i++ )
			buff[i] = toupper(buff[i]);
		writeline( handle, buff );
		if( !strcmp(buff,"end") || !strcmp(buff, "END") )
			break;
	}
}

int ProxyGetFile( int client_handle )
{
	char srcFile[80];
	char buff[BUFFSIZE];
	int srcHandle;

	/* reading file name: remote or local file to be accessed. */
	readline( client_handle, srcFile, 80 );
	printf("test1: %s\n", srcFile);	
/* if( (srcHandle = globus_gass_open( srcFile, O_RDONLY|O_NONBLOCK)) == -1 ) */

	if( (srcHandle = open( srcFile, O_RDONLY|O_NONBLOCK)) == -1 )
	{
	printf("Error: Unable to open file--%s for reading at proxy.\n", srcFile); 
		writeline( client_handle, "no");
		return FILE_DOES_NOT_EXIST;
	}
	else
		writeline( client_handle, "yes" );
	printf("test2\n");	
	/* read local file and send to requester by Sockets */
	FileCopy( srcHandle, client_handle );
	close( srcHandle );
/*	globus_gass_close( srcHandle ); */
	return SUCCESS;
}

int ProxyPutFile( int client_handle )
{
	char dstFile[80], buff[BUFFSIZE];
	int dstHandle;

	/* reading file name: for writing to remote or local */

	readline( client_handle, dstFile, 80 );

/*        if( (dstHandle =  globus_gass_open( dstFile, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU  )) == -1 ) */
        if( (dstHandle =  open( dstFile, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU  )) == -1 )
	{
		writeline( client_handle, "no");
		return FILE_WRITE_OPEN_FAIL;
	}
	else
		writeline( client_handle, "yes" );
        /* read local file and send to requester by Sockets */
        FileCopy( client_handle, dstHandle );
        close( dstHandle );
/*        globus_gass_close( dstHandle ); */
	return SUCCESS;
}

int GetMoleculeRequest( int client_handle )
{
	char db_filename[BUFFSIZE];
	char buff[BUFFSIZE];
	long mol_no;
	int ret;

	/* get datbasefile name */
   if( readline( client_handle, db_filename, BUFFSIZE ) == -1 )
	{
		printf("Client failed to send database file name.\n");
		return FALSE;
	}

	/* get molecule_no */
   if( readline( client_handle, buff, BUFFSIZE ) == -1 )
	{
		printf("Client failed to send molecule no.\n");
		return FALSE;
	}
	sscanf(buff, "%ld", &mol_no );

	/* Send Molecule record to the requested client */
	ret = GetMoleculeRecord(db_filename, mol_no, client_handle);

	if(cdb_server_verbose)
	{
		printf("Database: %s Molecule: %d, ", db_filename, mol_no );
		switch( ret )
		{
			case 0:
				printf("Molecule Found :-)\n");
				break;
			case 1:
				printf("Unable to open %s file :-(\n", db_filename);
				break;
			case 2:
				MakeLookupFileName( db_filename, buff );
				printf("Unable to open %s file :-(\n", buff);
				break;
			case 3:
				printf("Molecule NOT Found :-(\n");
				break;
		}
	}
	if( ret == 0)
		return TRUE;
	else
		return FALSE;
}
